/******************************************************************************
 Author:   Grant McKee, Solarbotics Ltd. (C) 2004
 Date:     Apr 2004
 Software: AVR-GCC 3.3.1 
 Hardware: ATMEGA8L at 1 Mhz int OSC
           
Description:
Minisumo/Linefollower program Ver 1.0

If outside edge sensors see black during startup the program will branch to sumo. 
If outside edge sensors see white during startup the program will branch to Linefollower.

Sumo mode is fairly basic:

- Wait 5 seconds before moving
- After 5 seconds go straight forward
- If a white line is detected on outside Left sensor
	* Reverse both motors briefly
	* Stop left motor
	* Reverse Right motor
	* After set time continue straight forward
- If a white line is detected on outside Right sensor
	* Reverse both motors briefly
	* Stop Right motor
	* Reverse Left motor
	* After set time continue straight forward
- If opponent detected on right Side
	* Stop Right motor
- If opponent detected on Left Side
	* Stop Left motor
- If both sensors detect opponent
	* Turn on both motors forward
	
Linefollower mode:

- Start immediately
- If center sensor sees black all is well- go straight forward
- If center right sensor sees black, make a gentle right turn by slowing down right motor
- If center left sensor sees black, make a gentle left turn by slowing down left motor
- If far right sensor sees black, make a very sharp right turn by reversing right motor
- If far left sensor sees black, make a very sharp left turn by reversing left motor

- If all is white, all is lost - go looking for the line!
 
*******************************************************************************/

#include <avr/io.h>

int value,left,mleft,middle,mright,right,channel,IRin; //Defined variables
long x,z; //Variables for delay timers

int ADCIN(int channel); //Function prototype for ADC function

#define thresh 128 //White line sensitivity higher is more sensitive (255 max)
#define start_time 150000 //Startup delay time constant 150000 ~ 1 sec (at clk = 1Mhz)
#define reverse_time 60000 //Time to reverse

int main(void) //Start of main
{
outp(0xFC, DDRD); //set 5 LED's on port D as outputs
outp(0xFF, DDRB); //set outputs to motor enables/direction

/**************************** Line sensor switch **********************/
	left = ADCIN(0);  //Read Left line sensor
	right = ADCIN(4);  //Read Right line sensor

/****************** Startup routine (am I Sumo or Linefollower) ********/
if ((right < thresh) & (left < thresh))  //If both left and right sensors see white
	{
		LINEFOLLOWER(); //Go to the linefollower loop
	}

SUMO();
}
/************************** 5 second startup **************************/
SUMO()
{
sbi(PORTD, 2);	//Turn on L1
for(x=0; x<start_time; x++); //Pause for a second
sbi(PORTD, 3); //Turn on L2
for(x=0; x<start_time; x++); //Pause for a second
sbi(PORTD, 4); //Turn on L3
for(x=0; x<start_time; x++); //Pause for a second
sbi(PORTD, 5); //Turn on L4
for(x=0; x<start_time; x++); //Pause for a second
sbi(PORTD, 6); //Turn on L5
for(x=0; x<start_time; x++); //Pause for a second

cbi(PORTD, 2); //Turn off L1
cbi(PORTD, 3); //Turn off L2
cbi(PORTD, 4); //Turn off L3
cbi(PORTD, 5); //Turn off L4
cbi(PORTD, 6); //Turn off L5
/************************* End 5 second startup **********************/

while(1) //Do this sumo loop forever
	{
	left = ADCIN(0);  //Read line sensor (Left)
	//mleft = ADCIN(1); // (Center Left) - Not used for Sumo
	//middle = ADCIN(2); // (Center) - Not used for Sumo
	//mright = ADCIN(3); // (Center Right) - Not used for Sumo
	right = ADCIN(4); //Read line sensor (Right)
	
	sbi(PORTB, 1);	//Enable left motor	
	sbi(PORTB, 2);	//Enable Right motor
	sbi(PORTB, 4);	//Forward left motor
	sbi(PORTB, 5);	//Forward Right motor

if(right < thresh) //Seeing white line on right sensor
	{
		for(x=0; x<6000; x++) //slam both motors into reverse to prevent drifting over the line
		{
		cbi(PORTB, 4); //Reverse left motor
		cbi(PORTB, 5); //Reverse right motor
		}
	
		for(x=0; x<reverse_time; x++) //Turn around with duration given by "reverse_time"
		{
		cbi(PORTB, 4); //Reverse left motor
		cbi(PORTB, 2); //Stop right motor
		}
	}

if(left < thresh) //Seeing white line on left sensor
	{
		for(x=0; x<6000; x++) //slam both motors into reverse to prevent drifting over the line
		{
		cbi(PORTB, 4); //Reverse left motor
		cbi(PORTB, 5); //Reverse right motor
		}
	
		for(x=0; x<reverse_time; x++)
		{
		cbi(PORTB, 1); //Stop left motor
		cbi(PORTB, 5); //Reverse right motor
		}
	}

IRin = inp(PIND); //Read inputs from port D
IRin = (IRin & 0x3); //Mask bits (1100000) PD0, PD1

if(IRin == 2) //See opponent with Right sensor
{
sbi(PORTD, 2); //Turn on L1
cbi(PORTD, 6); //Turn off L5
cbi(PORTB, 1);	//Disable Right motor
}

if(IRin == 1) //See opponent with Left sensor
{
sbi(PORTD, 6); //Turn on L5
cbi(PORTD, 2); //Turn off L1
cbi(PORTB, 2);	//Disable Left motor
}

if(IRin == 0) //Both sensors see opponent
{
sbi(PORTD, 2); //Turn on L1
sbi(PORTD, 6); //Turn on L5
sbi(PORTB, 2);	//Enable Left motor
sbi(PORTB, 1);	//Enable Right motor
}

if(IRin == 3) //Neither sensor sees opponent
{
cbi(PORTD, 2); //Turn on L1
cbi(PORTD, 6); //Turn on L5
}
}
}

/****************************** Linefollower start *******************************/
LINEFOLLOWER()
{
while(1) //Do loop forever
{
TCCR1A = BV(WGM10) | BV(COM1A1) | BV(COM1B1); //Setup PWM for left and right motors
TCCR1B = BV(CS10) | BV(WGM12); //ditto

	left = ADCIN(channel = 0);  //Read line sensors (Left)
	mleft = ADCIN(channel = 1); //(Middle Left)
	middle = ADCIN(channel = 2); //(Middle)
	mright = ADCIN(channel = 3); //(Middle Right)
	right = ADCIN(channel = 4); //(Right)
	
	
if(right < thresh) //Seeing white on right sensor
		{
		sbi(PORTD, 6);		//Turn on LED5	
		}
if(right > thresh) //Seeing black on right sensor
		for(z=0; z<200; z++)
	{
		cbi(PORTD, 6);		//Turn off LED5
		cbi(PORTB, 5);		//Left motor backward
	}
if(mright < thresh) //Seeing white on middle right sensor
		{
		sbi(PORTD, 5);		//Turn on LED4
		}
if(mright > thresh) //Seeing black on middle right sensor
		for(z=0; z<200; z++)
	{
		cbi(PORTD, 5);		//Turn off LED4
		OCR1B = 127;		//Slow down Right motor
	}
	
if(middle < thresh) //Seeing white on middle sensor
		{
		sbi(PORTD, 4);		//Turn on LED3	
		}
if(middle > thresh) //Seeing black on middle sensor
		{
		cbi(PORTD, 4);		//Turn off LED3
		sbi(PORTB, 4);		//Forward Right motor
		sbi(PORTB, 5);		//Forward Left motor
		OCR1B = 255;		//Set speed of Right motor
		OCR1A = 255;		//Set Speed of Left motor
		}
		
if(mleft < thresh) //Seeing white
		{
		sbi(PORTD, 3);		//Turn on LED2	
		}
if(mleft > thresh) //Seeing black
		for(z=0; z<200; z++)
	{
		cbi(PORTD, 3);		//Turn off LED2
		OCR1A = 127;		//Set Speed of Left motor
	}
if(left < thresh) //Seeing white
		{
		sbi(PORTD, 2);		//Turn on LED1
		}
if(left > thresh) //Seeing black
		for(z=0; z<200; z++)
	{
		cbi(PORTD, 2);		//Turn off LED1
		cbi(PORTB, 4);		//Reverse right motor
	}

}
}
	
/******************************** ADC function *********************************/
	int ADCIN(int channel)
	{
	//Left adjust result for 8 bit res  //Set reference to AVCC //Set channel
	ADMUX = BV(ADLAR) | BV(REFS0) | channel;

	//Enable ADC //Start Conversion
	ADCSRA = BV(ADEN) | BV(ADSC);
	
	for(x=0; x<10; x++); // Pause for channel change
	
	//Wait until conversion is complete
	loop_until_bit_is_clear(ADCSRA, BV(ADSC));
	
	//Write value to ADCH
	value = ADCH;
	
	return value;
    }
/************************************ END ***************************************/
